import stringify = require('json-stringify-safe')
import {uniqBy} from 'lodash'
import {AST, T_ANY} from './types/AST'
import {log} from './utils'

export function optimize(ast: AST, processed = new Map<AST, AST>()): AST {
  log('cyan', 'optimizer', ast, processed.has(ast) ? '(FROM CACHE)' : '')

  if (processed.has(ast)) {
    return processed.get(ast)!
  }

  processed.set(ast, ast)

  switch (ast.type) {
    case 'INTERFACE':
      return Object.assign(ast, {
        params: ast.params.map(_ => Object.assign(_, {ast: optimize(_.ast, processed)}))
      })
    case 'INTERSECTION':
    case 'UNION':
      // [A, B, C, Any] -> Any
      if (ast.params.some(_ => _.type === 'ANY')) {
        log('cyan', 'optimizer', ast, '<- T_ANY')
        return T_ANY
      }

      // [A, B, B] -> [A, B]
      const shouldTakeStandaloneNameIntoAccount = ast.params.filter(_ => _.standaloneName).length > 1
      ast.params = uniqBy(
        ast.params,
        _ => `
          ${_.type}-
          ${shouldTakeStandaloneNameIntoAccount ? _.standaloneName : ''}-
          ${stringify((_ as any).params)}
        `
      )

      return Object.assign(ast, {
        params: ast.params.map(_ => optimize(_, processed))
      })
    default:
      return ast
  }
}
